const {
    Before,
    BeforeAll,
    After,
    Given,
    Then,
    When,
} = require('@cucumber/cucumber');
const { GetConsent, DeleteConsent, personLookup, RegisterConsent, RegisterConsentCustody } = require('dataaccess/fgvhr');
const jq = require('node-jq');
const { expect } = require('../util');
const roles = new Map(require('../../config/roles.json'));


///////////////////////////////////////////////////////////////
//              Given statements                             //
///////////////////////////////////////////////////////////////

//tjek på om borger har en værge
Given('borger {string} er værge til borger {string}',
    async function (guardianCPR, patientCpr) {
        patientCpr = this.replaceVariables(patientCpr);

        const response = await personLookup(this.attach, patientCpr)
        const guardian =
            response.body.response.main.Envelope.Body.getPersonInformationOut
                .PersonInformationStructure.PersonCustodyStructure;

        let responseGuardian;
        if (guardian[1] != null) {
            guardian.forEach(custody => {
                if (custody.PersonCustodyRelation._text == 'vaerge') {
                    responseGuardian = custody.PersonCustodyIdentifier._text;
                }
            });
        } else {
            if (guardian.PersonCustodyRelation._text == 'vaerge') {
                responseGuardian = guardian.PersonCustodyIdentifier._text;
            }
        }
        expect(guardianCPR).to.eql(responseGuardian);
    });

//checker at borger er under (eksempelvis 60)
Given('borger {string} alder er {string} {int}',   async function (patientCpr, status, age) {
    patientCpr = this.replaceVariables(patientCpr);
    var responseAge;
    const response = await personLookup(this.attach, patientCpr)
    const responseBirth =
        response.body.response.main.Envelope.Body.getPersonInformationOut
            .PersonInformationStructure.RegularCPRPerson.PersonBirthDateStructure.BirthDate._text;

    let part = responseBirth.slice(0, 10);
    let date = new Date(part);

    if (new Date().getFullYear() - date.getFullYear() == 60) {
        if (new Date().getMonth() < date.getMonth()) {
            responseAge = new Date().getFullYear() - date.getFullYear() - 1;
        }
        else if (new Date().getMonth() == date.getMonth()) {
            if (new Date().getDate() < date.getDate()) {
                responseAge = new Date().getFullYear() - date.getFullYear() - 1;
            }
            else {
                responseAge = new Date().getFullYear() - date.getFullYear();
            }
        }
        else {
            responseAge = new Date().getFullYear() - date.getFullYear();
        }
    } else {
        responseAge = new Date().getFullYear() - date.getFullYear();
    }
    console.log(responseAge);
    switch (status) {
        case 'under':
            expect(responseAge).to.lessThan(age);
            break;
        case 'over':
            expect(responseAge).to.greaterThanOrEqual(age);
            break;
    }
});




//sikre at borgeren har en fravalgsregistrering med status draft
Given('fravalgsregistrering er oprettet for {string} af borger',
    async function (patientCpr) {
        patientId = this.replaceVariables(patientCpr);
        await RegisterConsent(this.attach, patientId);
    });

    Given(
        'fravalgsregistrering {string} for borger med {string}',
        { timeout: -1 },
        async function (Status, PatientCPR) {
            // Replacing CPR
            PatientCPR = this.replaceVariables(PatientCPR);
    
            // Checking if the patient has an FGVHR registration
            const hasFGVHR = await GetConsent(
                this.attach,
                PatientCPR
            )
    
            // Switch cases based on organ donor registration must or must not exist
            switch (Status) {
                case 'findes':
                    console.log(1);
                    // No fgvhr registration
                    if (hasFGVHR == 'inactive') {
                        console.log(2);
                        // Creating fgvhr registration
                        await RegisterConsent(this.attach, PatientCPR);
                    }
                    break;
                case 'findes ikke':
                    // Organ donor registration exists
                    if (hasOrganDonor == 'true') {
                        // Deleting organ donor registration
                        await DeleteConsent(this.attach, PatientCPR);
                    }
                    break;
                default:
                    // Throwing error if status is unknown
                    throw new Error(`Unknown status "${Status}"`);
            }
        }
    );


// sikre at borgeren har en inaktiv fravalgs registrering    
Given('fravalgsregistrering er slettet for {string} af borger',
    async function (patientCpr) {
        patientId = this.replaceVariables(patientCpr);
        await DeleteConsent(this.attach, patientId);
    });

Given('{string} registrerer fravalg af genoplivning for borger {string}',
    async function (custodyCpr, patientCpr) {
        patientId = this.replaceVariables(patientCpr);
        custodyId = this.replaceVariables(custodyCpr);

        const response = await RegisterConsentCustody(this.attach, custodyId, patientId);
        this.responses.push(response);
    });

//Ny dato gemems for dagens dato, da funktionaliteten ikke tog i mod formatet: '2023-10-25T13:52:42Z', men tager imod '2023-10-25'
Given('datasæt {string} med felterne for FGVHR', function (name, dataTable) {
    let today = new Date();
    var datetime = today.getFullYear() + "-" + (today.getMonth() + 1) + "-" + today.getDate();

    let data = this.parseDataTable(dataTable);
    data.signingDate = datetime;
    this.datasets.set(name, data);
    console.log(name, data);
});

///////////////////////////////////////////////////////////////
//              Then statements                             //
///////////////////////////////////////////////////////////////

Then('Oprettet dato for registreringen er {string} i {string}',
    async function (createdDate, responseName) {
        let response = this.variables.get(responseName);
        createdDate = this.replaceVariables(createdDate).slice(0, 10);
        let createdDateInResponse = response.body.response.main.Envelope.Header.Security.Timestamp.Created._text
        let dateResponse = createdDateInResponse.slice(0, 10);
        expect(dateResponse).to.eql(createdDate)
    });


When('der {string} en FGVHR registrering for borger med cpr {string} med status {string} i {string}',
    async function (status, patientCpr, registrering, responseName) {
        let response = this.variables.get(responseName);
        patientCpr = this.replaceVariables(patientCpr);
        registrering = this.replaceVariables(registrering);

        switch (status) {
            case 'findes':
                let registreringsStatus = response.body.response.main.Envelope.Body.Consent.status.__value
                console.log(registreringsStatus);
                expect(registreringsStatus).to.eql(registrering)

                let patientCprISvar =
                    response.body.response.main.Envelope.Body.Consent.grantor.identifier.value.__value
                console.log(patientCprISvar);
                expect(patientCprISvar).to.eql(patientCpr)
                break

            case 'findes ikke':
                console.log("findes ikke");
                break
            default:
                break
        }
    });




