const {
    Before,
    BeforeAll,
    After,
    Given,
    Then,
    When,
} = require('@cucumber/cucumber');
const jq = require('node-jq');
const { expect } = require('../util');
const roles = new Map(require('../../config/roles.json'));

Then('retursvar indeholder {string} lig med {string}', async function (element, expectedValue) {

    element = this.replaceVariables(element);
    expectedValue = this.replaceVariables(expectedValue);
    let response;

    if (this.getLastResponse().body.response.main.Envelope.Body.getPersonWithHealthCareInformationOut == undefined) {
        response =
            this.getLastResponse().body.response.main.Envelope.Body
                .getPersonInformationOut.PersonInformationStructure;
        let infoValue;
        switch (element) {
            case 'PersonGivenName':
                infoValue = response.RegularCPRPerson.SimpleCPRPerson.PersonNameStructure.PersonGivenName._text;
                break;
            case 'PersonMiddleName':
                infoValue = response.RegularCPRPerson.SimpleCPRPerson.PersonNameStructure.PersonMiddleName._text;
                break;
            case 'PersonSurnameName':
                infoValue = response.RegularCPRPerson.SimpleCPRPerson.PersonNameStructure.PersonSurnameName._text;
                break;
            case 'PersonCivilRegistrationIdentifier':
                infoValue = response.RegularCPRPerson.SimpleCPRPerson.PersonCivilRegistrationIdentifier._text;
                break;
            case 'PersonNameForAddressingName':
                infoValue = response.RegularCPRPerson.PersonNameForAddressingName._text;
                break;
            case 'PersonGenderCode':
                infoValue = response.RegularCPRPerson.PersonGenderCode._text;
                break;
            case 'PersonCivilRegistrationStatusCode':
                infoValue = response.RegularCPRPerson.PersonCivilRegistrationStatusStructure.PersonCivilRegistrationStatusCode._text;
                break;
            case 'StreetName':
                infoValue = response.PersonAddressStructure.AddressComplete.AddressPostal.StreetName._text;
                break;
            case 'StreetNameForAddressingName':
                infoValue = response.PersonAddressStructure.AddressComplete.AddressPostal.StreetNameForAddressingName._text;
                break;
            case 'MunicipalityCode':
                infoValue = response.PersonAddressStructure.AddressComplete.AddressAccess.MunicipalityCode._text;
                break;
            case 'StreetCode':
                infoValue = response.PersonAddressStructure.AddressComplete.AddressAccess.StreetCode._text;
                break;
            case 'StreetBuildingIdentifier':
                infoValue = response.PersonAddressStructure.AddressComplete.AddressAccess.StreetBuildingIdentifier._text;
                break;
            case 'PostCodeIdentifier':
                infoValue = response.PersonAddressStructure.AddressComplete.AddressPostal.PostCodeIdentifier._text;
                break;
            case 'DistrictName':
                infoValue = response.PersonAddressStructure.AddressComplete.AddressPostal.DistrictName._text;
                break;
            case 'CountyCode':
                infoValue = response.PersonAddressStructure.CountyCode._text;
                break;
            case 'PersonInformationProtectionIndicator':
                infoValue = response.RegularCPRPerson.PersonInformationProtectionIndicator._text;
                break;
        }
        expect(expectedValue).to.equal(infoValue);
    } else {
        response = this.getLastResponse().body.response.main.Envelope.Body.getPersonWithHealthCareInformationOut
            .PersonWithHealthCareInformationStructure.PersonHealthCareInformationStructure;
        let value;
        switch (element) {
            case 'AssociatedGeneralPractitionerIdentifier':
                value = response.CurrentGeneralPractitionerStructure.AssociatedGeneralPractitionerStructure.AssociatedGeneralPractitionerIdentifier._text;
                break;
            case 'AssociatedGeneralPractitionerOrganisationName':
                value = response.CurrentGeneralPractitionerStructure.AssociatedGeneralPractitionerStructure.AssociatedGeneralPractitionerOrganisationName._text;
                break;
            case 'StandardAddressIdentifier':
                value = response.CurrentGeneralPractitionerStructure.AssociatedGeneralPractitionerStructure.StandardAddressIdentifier._text;
                break;
            case 'PostCodeIdentifier':
                value = response.CurrentGeneralPractitionerStructure.AssociatedGeneralPractitionerStructure.PostCodeIdentifier._text;
                break;
            case 'DistrictName':
                value = response.CurrentGeneralPractitionerStructure.AssociatedGeneralPractitionerStructure.DistrictName._text;
                break;
            case 'TelephoneSubscriberIdentifier':
                value = response.CurrentGeneralPractitionerStructure.AssociatedGeneralPractitionerStructure.TelephoneSubscriberIdentifier._text;
                break;
            case 'EmailAddressIdentifier':
                value = response.CurrentGeneralPractitionerStructure.AssociatedGeneralPractitionerStructure.EmailAddressIdentifier._text;
                break;
            case 'PublicHealthInsuranceGroupIdentifier':
                value = response.PersonPublicHealthInsurance.PublicHealthInsuranceGroupIdentifier._text;
                break;
        }
        expect(expectedValue).to.equal(value);
    }
});

Then('retursvar indeholder værge med cpr {string} med relation {string}', function (cpr, relation) {
    cpr = this.replaceVariables(cpr);
    relation = this.replaceVariables(relation);

    const response =
        this.getLastResponse().body.response.main.Envelope.Body
            .getPersonInformationOut.PersonInformationStructure.PersonCustodyStructure;

    let custodyId
    let custodyRelation;
    if (response.length > 0) {
        response.forEach(custody => {
            if (custody.PersonCustodyIdentifier._text == cpr) {
                custodyId = custody.PersonCustodyIdentifier._text;
                custodyRelation = custody.PersonCustodyRelation._text
            }
        });
    }
    else {
        custodyId = response.PersonCustodyIdentifier._text;
        custodyRelation = response.PersonCustodyRelation._text;
    }
    expect(custodyId).to.equal(cpr);
    expect(custodyRelation).to.equal(relation);
});

Then('retursvar indeholder myndling med cpr {string} med relation {string}', function (cpr, relation) {
    cpr = this.replaceVariables(cpr);
    relation = this.replaceVariables(relation);

    const response =
        this.getLastResponse().body.response.main.Envelope.Body
            .getPersonInformationOut.PersonInformationStructure.PersonWardStructure;

    let wardId
    let wardRelation;
    if (response.length > 0) {
        response.forEach(ward => {
            if (ward.PersonWardIdentifier._text == cpr) {
                wardId = ward.PersonWardIdentifier._text;
                wardRelation = ward.PersonWardRelation._text
            }
        });
    }
    else {
        wardId = response.PersonWardIdentifier._text;
        wardRelation = response.PersonWardRelation._text;
    }
    expect(wardId).to.equal(cpr);
    expect(wardRelation).to.equal(relation);
});


